# coding: utf-8
"""
Export AFM
=============================

Command per utilizzo::

  dj export_afm

:-c: specify the class model using format <app_name>.<model_name> (ex: dj export_model_xls -c 'content.Car')
:-f: specify string filter using a json dictionary (ex: -f '{"status":"ok"}')
:-l: specify list of fields using a json dictionary (ex: -l '["id","status","infocar"]')
:-L: limit queryset to x elements
:-o: required: output path and file name (ex: /tmp/export.csv)
:-t: type of export. CSV is default. at the moment only CSV is supported (ex: -t csv)
:-s: fields for select_related django orm parameter
:-p: fields for prefetch_related django orm parameter
"""
import json
from django.utils.translation import ugettext_lazy as _
from django.db.models import get_model
from django.core.management.base import BaseCommand, CommandError

from optparse import make_option

from jmb.core.admin.options import ExportData


class Command(BaseCommand):
    """Exports items from given model
    """

    option_list = BaseCommand.option_list + (
        make_option(
            '-c', '--class_name', dest='class_name', help='name of the class for which export data', default=""
        ),
        make_option(
            '-f', '--filter', dest='filters', help='filters queryset on values specified in dictionary', default="{}"
        ),
        make_option(
            '-l', '--list', dest='fields', help='list of fields/properties of the model to export', default="[]"
        ),
        make_option(
            '-L', '--limit',
        ),
        make_option(
            '-o', '--output',
            help='filename path',
        ),
        make_option(
            '-s', '--select_related', default="[]",
            help='list of field to use as select_related parameter'
        ),
        make_option(
            '-p', '--prefetch_related', default="[]",
            help='list of field to use as prefetch_related parameter'
        ),
        make_option(
            '-t', '--type', default="csv",
            help='type of output'
        ),
    )

    def handle(self, *args, **opts):
        model = self.get_class_model(opts.get('class_name'))

        filters = self.get_filters(opts.get('filters'))

        fields = self.get_fields(opts.get('fields'))

        select_related = self.get_select_related(opts.get('select_related'))

        prefetch_related = self.get_prefetch_related(opts.get('prefetch_related'))

        limit = opts.get('limit')

        output = opts.get('output')

        output_type = opts.get('type')

        self.export_model(
            model=model, filters=filters,
            fields=fields, output=output,
            limit=limit, output_type=output_type,
            select_related=select_related,
            prefetch_related=prefetch_related,
        )

    @staticmethod
    def get_class_model(class_name):
        """ Get class model from string
        """
        try:
            return get_model(*class_name.split('.', 1))
        except Exception:
            raise CommandError(
                _("""Model with label {0} could not be found.
                Please specify a model using format <app_name>.<model_name>""".format(class_name))
            )

    @staticmethod
    def get_fields(fields_string):
        """ Get the fields list from json formatted string
        """
        try:
            return json.loads(fields_string)
        except Exception:
            raise CommandError(
                _("Filter string option {0} must be a correct json list format.".format(fields_string))
            )

    @staticmethod
    def get_filters(filters_string):
        """ Get the filters list from json formatted string
        """
        try:
            return json.loads(filters_string)
        except Exception:
            raise CommandError(
                _("Filter string option {0} must be a correct json dict format.".format(filters_string))
            )

    @staticmethod
    def get_select_related(select):
        """ Get the filters list from json formatted string
        """
        try:
            return json.loads(select)
        except Exception:
            raise CommandError(
                _("Filter string option %s must be a correct json list format.".format(select))
            )

    @staticmethod
    def get_prefetch_related(prefetch):
        """ Get the filters list from json formatted string
        """
        try:
            return json.loads(prefetch)
        except Exception:
            raise CommandError(
                _("Filter string option %s must be a correct json list format.".format(prefetch))
            )

    @staticmethod
    def get_queryset(model, filters, select_related, prefetch_related, limit):
        """ Get the queryset of items from given model
        """
        qs = model.objects.filter(**filters)
        if select_related:
            qs = qs.select_related(*select_related)
        if prefetch_related:
            qs = qs.prefetch_related(*prefetch_related)

        if limit:
            qs = qs[:limit]
        return qs

    def export_model(self, model, filters, select_related, prefetch_related, limit, fields, output, output_type):
        """ Export the document as cvs file
        """
        qs = self.get_queryset(model, filters, select_related, prefetch_related, limit)
        export = ExportData(model=model, queryset=qs, fields_list=fields)
        return export.export(filename=output, type=output_type)
